module TaskScheduler
  module Scheduler
    TIMEFRAMES = %w[hours days months].freeze
    
    def self.included(base)
      base.class_eval do
        after_update :update_scheduled_task, if: 'OS.windows?'
        after_create :create_scheduled_task, if: 'OS.windows?'
        
        validates_presence_of :scheduler, :task_name, :rake_task_name, :import_at,
                              :import_time, :repeat_time, :repeat_type, :username,
                              :password
      end
    end
    
    private
    
    def create_scheduled_task
      scheduler = WindowsScheduler.new(task_name, rake_task_name)
      task = scheduler.create_task(import_at, import_time, repeat_type, repeat_time, username, password, batch_params)
      task.run
    end
    
    def update_scheduled_task
      scheduler = WindowsScheduler.new(task_name, rake_task_name)
      task = scheduler.update_task(import_at, import_time, repeat_type, repeat_time, username, password, batch_params)
      task.run
    end
  end
end
