module TaskScheduler
  class WindowsScheduler
    attr_reader :task_name, :rake_task, :import_at, :repeat_type, :repeat_time
    
    def initialize(task_name, rake_task)
      @task_name = task_name
      @rake_task = rake_task
    end
    
    def run_task(system_ip = nil)
      io = StringIO.new
      io << 'SchTasks /Run '
      io << "/S #{system_ip} " if system_ip # run a task on a different system
      io << "/TN '#{task_name}' "
      io
    end
    
    # creates a windows scheduled task via command line
    #
    def create_task(import_at, import_time, repeat_type, repeat_time, username, password, bat_params = nil)
      io = StringIO.new
      io << 'SchTasks /Create '
      io << "/RU #{username} /RP #{password} " 
      if repeat_type && repeat_time
        io << "/SC #{get_schedule_type(repeat_type)} " # schedule frequency
        io << "/MO #{repeat_time} " # time between runs
      end
      io << "/TN '#{task_name}' " # name of the task
      io << "/TR '#{build_bat_params(bat_params)}' " # what to execute when task runs
      io << "/ST '#{import_time.strftime('%H:%M')}' " if import_time # start time
      io << "/SD '#{import_at.strftime('%m/%d/%Y')}'" if import_at # start date
      io
    end
    
    # updates a windows scheuled task via the command line
    # 
    def update_task(import_at, import_time, repeat_type, repeat_time, username, password, bat_params = nil)
      delete_task&.run
      create_task(import_at, import_time, repeat_type, repeat_time, username, password, bat_params)
    end
    
    def delete_task
      io = StringIO.new
      io << "SchTasks /Delete /TN '#{task_name}' /F"
    end
    
    private
    
    def build_bat_params(bat_params)
      io = StringIO.new
      io << File.join(File.expand_path('..', File.dirname(__FILE__)), 'task_runner.bat')
      io << " #{Rails.root} " # first param so the bat file knows where to cd into
      io << "TASKNAME=#{rake_task} "
      
      return io.string unless bat_params
      
      bat_params.each do |param_name, value|
        io << "#{param_name.upcase}=#{value} "
      end
      io.string
    end
    
    def get_schedule_type(type)
      if type == 'minutes'
        'MINUTE'
      elsif type == 'hours'
        'HOURLY'
      elsif type == 'days'
        'DAILY'
      end
    end
  end
end

class StringIO
  # I built this method because I couldn't test with the system method in the create_task method
  def run
    system(string)
  end
end
